//
//  AAICrashReporter.hpp
//  AAICrash
//
//  Created by advance on 2024/5/14.
//

#ifndef AAICrashReporter_hpp
#define AAICrashReporter_hpp

#include <stdio.h>
#import <UIKit/UIKit.h>
#include <string>
#include <mutex>
#include <iostream>
#include <list>
#include "AAICrashConstant.h"
#include <variant>

namespace aai
{
namespace crash
{

extern volatile sig_atomic_t requiresAsyncSafety;

enum class ValueType {
    UNDEFINE,
    INT,
    LONG,
    FLOAT,
    DOUBLE,
    PTR
};

using MyVariant = std::variant<int, long, float, double, void *>;

struct DataItem {
public:
    ValueType valueType = ValueType::UNDEFINE;
    size_t len = 0;
    MyVariant value;
    
    bool requiresAsyncSafety = false;
    
    void initialize(const char* _Nullable str, size_t len, bool requiresAsyncSafety) {
        this->valueType = ValueType::PTR;
        this->value = nullptr;
        this->len = len;
        this->requiresAsyncSafety = requiresAsyncSafety;
        
        if (requiresAsyncSafety) {
            this->value = (void *)str;
        } else {
            AAI_CRASH_ASSERT_TRUE(aai::crash::requiresAsyncSafety, "Avoid call initialize(malloc mode) on DataItem objects within signal handlers! %p", this);
            
            // Copy content
            if (len > 0 && str != NULL) {
                void *data = malloc(len * sizeof(char));
                if (data) {
                    memcpy(data, str, len);
                }
                
                this->value = data;
            } else {
                this->value = nullptr;
                this->len = 0;
            }
            
        }
    }
    
public:
    DataItem() = default;
    
    DataItem(ValueType valueType, MyVariant value): valueType(valueType) {
        assert(valueType != ValueType::PTR);
        
        #define AAI_CR_PROCESS_VALUE_TYPE(VT, T)  \
                    case ValueType::VT: {         \
                        this->len = sizeof(T);    \
                        if (auto p = std::get_if<T>(&value)) { this->value = *p; } \
                        break; \
                    }
        switch (valueType) {
                AAI_CR_PROCESS_VALUE_TYPE(INT, int)
                AAI_CR_PROCESS_VALUE_TYPE(LONG, long)
                AAI_CR_PROCESS_VALUE_TYPE(FLOAT, float)
                AAI_CR_PROCESS_VALUE_TYPE(DOUBLE, double)
                
            default:
                break;
        }
    }
    
    DataItem(const char * _Nullable str, size_t len, bool requiresAsyncSafety) {
        // Deep copy
        initialize(str, len, requiresAsyncSafety);
    }
    
    DataItem(std::string str) {
        // Deep copy
        initialize(str.data(), str.size(), false);
    }
    
    DataItem& operator=(DataItem&& other) noexcept {
        // Here we don't check valueType because DataItem() mean a invalid item,
        // Generally it's used to initialize a empty KVDataItem()
        if (this != &other) {
            len = other.len;
            valueType = other.valueType;
            requiresAsyncSafety = other.requiresAsyncSafety;
            
            if (other.valueType != ValueType::PTR) {
                value = other.value;
            } else {
                // 释放当前对象的内存
                if (auto p = getData()) {
                    AAI_CRASH_ASSERT_TRUE(aai::crash::requiresAsyncSafety, "Avoid call free within signal handlers!");
                    free(p);
                }
                value = other.value; // 接管 other 的内存
                other.value = nullptr;  // 确保 other 不再拥有内存
            }
            
        }
        return *this;
    }
    
    DataItem& operator=(const DataItem& other) {
        if (this != &other) {
            valueType = other.valueType;
            
            requiresAsyncSafety = other.requiresAsyncSafety;
            if (other.valueType != ValueType::PTR) {
                len = other.len;
                value = other.value;
                return *this;
            } else {
                if (other.requiresAsyncSafety) {
                    len = other.len;
                    value = other.value;
                    requiresAsyncSafety = other.requiresAsyncSafety;
                    return *this;
                }
                AAI_CRASH_ASSERT_TRUE(aai::crash::requiresAsyncSafety, "Warning: Avoid using 'operator=' on DataItem objects within signal handlers!");
                
                if (auto p = std::get_if<void *>(&value)) {
                    if (*p != nullptr) {
                        free(*p);  // 释放当前对象的内存
                    }
                }
                
                bool getPtrSucceed = false;
                if (other.len > 0) {
                    void *data = malloc(other.len);
                    if (data != nullptr) {
                        auto otherPtr = other.getData();
                        if (otherPtr != nullptr) {
                            memcpy(data, otherPtr, other.len);
                            value = data;
                            len = other.len;
                            getPtrSucceed = true;
                        } else {
                            free(data);
                        }
                    }
                }
                if (!getPtrSucceed) {
                    value = nullptr;
                    len = 0;
                }

            }

        }
        return *this;
    }
    
    bool isValid () {
        if (valueType == ValueType::UNDEFINE) {
            return false;
        }
        
        if (valueType == ValueType::PTR) {
            if (len == 0) {
                return false;
            } else {
                return getData() != nullptr;
            }
        } else {
            return true;
        }
    }
    
    void* _Nullable getData() const {
        if (auto p = std::get_if<void *>(&value)) {
            return *p;
        }
        return nullptr;
    }
    
    size_t getSize() const {
        return len;
    }
    
    ~DataItem() {
        void *data = getData();
        
        if (valueType == ValueType::PTR) {
            if (!requiresAsyncSafety) {
                if (data) {
                    AAI_CRASH_ASSERT_TRUE(aai::crash::requiresAsyncSafety, "Warning: Avoid free heap DataItem objects within signal handlers! %p", this);
                    
                    free(data);
                    data = nullptr;
                }
            }
        }
    }
    
    std::string toString() {
        void *data = getData();
        if (data != nullptr) {
            return std::string((char *)data, len);
        } else {
            return "";
        }
    }
    
};

struct KVDataItem {
    DataItem key;
    DataItem value;
    
    KVDataItem() = default;
    KVDataItem(std::string key, std::string value) {
        this->key = DataItem(key);
        this->value = DataItem(value);
    }
    
    KVDataItem(std::string key, const DataItem &value) {
        this->key = DataItem(key);
        this->value = value;
    }
    
    bool isValid() {
        return key.isValid() && value.isValid();
    }
};

struct Writer {
private:
    char * _Nullable basePath = nullptr;
    int fd = -1;
    
    bool writeData(const void * _Nonnull buffer, size_t len) {
        const char *pos = (const char *)buffer;
        size_t byteToWrite = len;
        while (byteToWrite > 0) {
            size_t byteWritten = write(fd, pos, byteToWrite);
            if (byteWritten == -1) {
                return false;
            }
            byteToWrite -= byteWritten;
            pos += byteWritten;
        }
        return true;
    }
    
    bool writeDataItem(DataItem &item) {
        if (fd != -1) {
            // 1. Save value type first. We don't allow write the UNDEFINE type item.
            assert(item.valueType != ValueType::UNDEFINE);
            int valueType = static_cast<int>(item.valueType);
            bool r = writeData(&valueType, sizeof(int));
            if (!r) return false;
            
            // 2. Save value length
            r = writeData(&(item.len), sizeof(size_t));
            if (!r) return false;
            
            // 3. Save value data
            #define AAI_CR_GET_THEN_WRITE(VT, T) \
                        case ValueType::VT: { \
                            if (auto p = std::get_if<T>(&item.value)) { \
                                r = writeData(p, sizeof(T)); \
                                if (!r) return false; \
                            } \
                            break; \
                        }

            switch (item.valueType) {
                case ValueType::PTR: {
                        // 如果是指针类型，再存内容
                        if (item.len > 0) {
                            void *p = item.getData();
                            if (p != nullptr) {
                                r = writeData(p, item.len * sizeof(char));
                                if (!r) return false;
                            }
                        }
                        break;
                    }
                    // 存基础数据类型
                    AAI_CR_GET_THEN_WRITE(INT, int)
                    AAI_CR_GET_THEN_WRITE(LONG, long)
                    AAI_CR_GET_THEN_WRITE(FLOAT, float)
                    AAI_CR_GET_THEN_WRITE(DOUBLE, double)
                default:
                    break;
            }
            
            // 4.
            bool requiresAsyncSafety = item.requiresAsyncSafety;
            writeData(&requiresAsyncSafety, sizeof(bool));
            if (!r) return false;
            
            return true;
        }
        return false;
    }
public:
    void setLogFileBasePath(const char * _Nullable basePath) {
        if (this->basePath) {
            free(this->basePath);
            this->basePath = nullptr;
        }
        if (basePath != nullptr) {
            this->basePath = strdup(basePath);
        }
    }
    
    // async signal safe
    bool writeKVUserData(KVDataItem &item) {
        if (fd != -1) {
            if (writeDataItem(item.key) && writeDataItem(item.value)) {
                return true;
            }
        }
        return false;
    }
    
    const char * _Nullable getBasePath() {
        return basePath;
    }
    
    inline void * _Nullable getRawPtr(MyVariant &var) {
        if (auto p = std::get_if<void *>(&var)) {
            return *p;
        }
        return nullptr;
    }
    
    std::shared_ptr<DataItem> readSingleDataItem() {
        if (fd == -1) return nullptr;
        
        // Read value type
        int rawValueType;
        if (read(fd, &rawValueType, sizeof(int)) < sizeof(int)) {
            return nullptr;
        }
        
        ValueType valueType = static_cast<ValueType>(rawValueType);
        if (valueType == ValueType::UNDEFINE) {
        #if DEBUG
            AAI_CRASH_CLOG("❌ValueType::UNDEFINE found! Read file offset:%lld", lseek(fd, 0, SEEK_CUR));
        #endif
            return nullptr;
        }
        
        // Read value length
        size_t len = 0;
        if (read(fd, &len, sizeof(size_t)) < sizeof(size_t)) {
            return nullptr;
        }
        
        #define AAI_CR_DECODE_FROM_FILE(VT, T) \
            case ValueType::VT: { \
                T tmpValue; \
                read(fd, &tmpValue, sizeof(T)); \
                var = tmpValue; \
                break; \
            }
        
        // Read value data
        MyVariant var;
        switch (valueType) {
            case ValueType::PTR: {
                // 如果是指针类型，再取内容
                if (len > 0) {
                    size_t readLen = len * sizeof(char);
                    void *value = malloc(readLen);
                    if (read(fd, value, readLen) < len) {
                        free(value);
                        return nullptr;
                    } else {
                        var = value;
                    }
                }
                break;
            }
            AAI_CR_DECODE_FROM_FILE(INT, int)
            AAI_CR_DECODE_FROM_FILE(LONG, long)
            AAI_CR_DECODE_FROM_FILE(FLOAT, float)
            AAI_CR_DECODE_FROM_FILE(DOUBLE, double)
            default:
                break;
        }
        
        // Read requiresAsyncSafety
        bool requiresAsyncSafety = false;
        if (read(fd, &requiresAsyncSafety, sizeof(bool)) < sizeof(bool)) {
            return nullptr;
        }
        
        if (valueType == ValueType::PTR) {
            if (auto p = getRawPtr(var)) {
                return std::make_shared<DataItem>((const char *)p, len, false);
            } else {
                return std::make_shared<DataItem>(nullptr, 0, false);
            }
        } else {
            return std::make_shared<DataItem>(valueType, var);
        }
    }
    
    std::list<std::shared_ptr<DataItem>> deserialize(int maxCount = 0) {
        std::list<std::shared_ptr<DataItem>> list;
        int count = 0;
        
        while (auto item = readSingleDataItem()) {
            list.push_back(item);
            
            if (maxCount > 0 && (++count) >= maxCount) {
                break;
            }
            AAI_CRASH_CLOG("Read dateItem count:%d", count);
        }
        return list;
    }
    
    // async signal safe
    bool openFile(const char * _Nonnull fileName, int flag = O_RDWR|O_CREAT|O_APPEND) {
        assert(fileName != nullptr);
        assert(basePath != nullptr);
        if (fileName == nullptr) return false;
        if (basePath == nullptr) return false;
        
        char fullPath[200] = {0};
        strcat(fullPath, basePath);
        strcat(fullPath, "/");
        strcat(fullPath, fileName);
        
        this->fd = open(fullPath, flag, 0644);
        if (this->fd != -1) {
            return true;
        } else {
            AAI_CRASH_CLOG("open file failed:%s", strerror(errno));
            return false;
        }
    }
    
    bool openFile(const char * _Nonnull fileName, const char * _Nonnull basePath, int flag) {
        setLogFileBasePath(basePath);
        return openFile(fileName, flag);
    }
    
    // async signal safe
    void closeFile() {
        if (this->fd != -1) {
            close(this->fd);
        }
    }
};


struct Reporter {
private:
    std::mutex mtx;
    
    uint64_t crashTimeMs = 0;
    char * _Nullable crashType = nullptr;
    uintptr_t instructionAddress = 0;
    
    const static size_t MAX_USER_DATA_LEN = 150;
    size_t kvDataLen = 0;
    KVDataItem kvUserData[MAX_USER_DATA_LEN];
    
    const char* _Nonnull logFileName = "aai_report.crash";
    
    Writer writer;
    
    void addDataItemFromUserDefault(NSBundle * _Nonnull bundle, NSString * _Nonnull dictKey, std::string dataItemKey) {
        NSString *value = [bundle objectForInfoDictionaryKey:dictKey];
        
        if (value) {
            KVDataItem dataItem(dataItemKey, std::string(value.UTF8String));
            if (dataItem.isValid()) {
                appendKVItem(dataItem);
            }
        }
    }
    
    void initAppInfo() {
        NSBundle *mainBundle = [NSBundle mainBundle];
        // AppInfo
        addDataItemFromUserDefault(mainBundle, @"CFBundleIdentifier", "appId");
        addDataItemFromUserDefault(mainBundle, @"CFBundleShortVersionString", "appVersion");
        addDataItemFromUserDefault(mainBundle, @"CFBundleName", "appName");
        
        // System version
        NSString *sysVersionStr = [UIDevice currentDevice].systemVersion;
        NSUInteger sysVersionStrSize = [sysVersionStr lengthOfBytesUsingEncoding:NSUTF8StringEncoding];
        KVDataItem sysVersionItem("sysVersion", std::string(sysVersionStr.UTF8String, sysVersionStrSize));
        appendKVItem(sysVersionItem);
    }
    
    // async signal safe
    void asyncSafeFlush() {
        for (int i = 0; i < kvDataLen; i++) {
            if (!writer.writeKVUserData(kvUserData[i])) {
                break;
            } else {
                // If we decrease kvDataLen, then if we call ayncSafeAppendKVItem,
                // will cause heap DateItem dealloc, which is not async safe
                // --kvDataLen;
            }
        }
    }
public:
    void clearKVUserData() noexcept {
        // Reset all data
        for (int i = 0; i < kvDataLen; i++) {
            kvUserData[i] = KVDataItem();
        }
        kvDataLen = 0;
    }
    
    void initReporter(NSString * _Nullable basePath) {
        reset();
        
        initAppInfo();
        
        // Create dir
        if (basePath) {
            writer.setLogFileBasePath(basePath.UTF8String);
        } else {
            NSString *basePath = defaultFileBasePath();
            writer.setLogFileBasePath(basePath.UTF8String);
        }
        
        NSString *dirPath = getReportFileBasePath();
        NSFileManager *fm = [NSFileManager defaultManager];
        if (![fm fileExistsAtPath:dirPath] &&
               ![fm createDirectoryAtPath:dirPath withIntermediateDirectories: YES attributes:nil error: nil])
        {
            return;
        }
    }
    
    NSString * _Nonnull defaultFileBasePath() {
        NSString *docPath = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES).firstObject;
        NSString *dirPath = [docPath stringByAppendingPathComponent: @"ai.advance.crash.data"];
        return dirPath;
    }
    
    NSString* _Nonnull getReportFileBasePath() {
        if (writer.getBasePath() == nullptr) {
            return defaultFileBasePath();
        } else {
            const char *cBasePath = writer.getBasePath();
            return [[NSString alloc] initWithCString:cBasePath encoding:NSUTF8StringEncoding];
        }
    }
    
    NSString * _Nonnull getReportFilePath() {
        NSString *fileName = [NSString stringWithCString:logFileName encoding:NSUTF8StringEncoding];
        return [getReportFileBasePath() stringByAppendingPathComponent:fileName];
    }
    
    size_t asyncSafeGetKVItemSize() {
        return kvDataLen;
    }
    
    bool appendKVItem(const KVDataItem &kvData) {
        std::lock_guard<std::mutex> lock(mtx);
        
        if (MAX_USER_DATA_LEN > kvDataLen) {
            kvUserData[kvDataLen] = kvData;
            ++kvDataLen;
            return true;
        }
        return false;
    }
    
    bool asyncSafeAppendKVItem(const KVDataItem &kvData) {
        if (MAX_USER_DATA_LEN > kvDataLen) {
            kvUserData[kvDataLen] = kvData;
            ++kvDataLen;
            return true;
        }
        return false;
    }
    
    // async signal safe
    bool asyncSafeOpenFile() {
        return writer.openFile(logFileName);
    }
    
    // async signal safe
    void asyncSafeCloseFile() {
        asyncSafeFlush();
        writer.closeFile();
    }
    
    void reset() {
        std::lock_guard<std::mutex> lock(mtx);
        clearKVUserData();
        writer.setLogFileBasePath(nullptr);
    }
    
    NSObject* _Nullable objcValue(DataItem * _Nonnull item) {
        if (item->valueType == ValueType::PTR) {
            NSString *str = [[NSString alloc] initWithBytes:item->getData() length:item->getSize() encoding:NSUTF8StringEncoding];
            AAI_CRASH_CLOG("[%s]", item->toString().c_str());
            return str;
        } else if (item->valueType == ValueType::INT){
            if (auto p = std::get_if<int>(&item->value)) {
                int tmpValue = *p;
                return @(tmpValue);
            }
        } else if (item->valueType == ValueType::LONG){
            if (auto p = std::get_if<long>(&item->value)) {
                long tmpValue = *p;
                return @(tmpValue);
            }
        } else if (item->valueType == ValueType::DOUBLE){
            if (auto p = std::get_if<double>(&item->value)) {
                double tmpValue = *p;
                return @(tmpValue);
            }
        } else if (item->valueType == ValueType::FLOAT){
            if (auto p = std::get_if<float>(&item->value)) {
                float tmpValue = *p;
                return @(tmpValue);
            }
        }
        return nullptr;
    }
    
    NSObject* _Nullable objcValue(DataItem &item) {
        if (item.valueType == ValueType::PTR) {
            NSString *str = [[NSString alloc] initWithBytes:item.getData() length:item.getSize() encoding:NSUTF8StringEncoding];
            AAI_CRASH_CLOG("[%s]", item.toString().c_str());
            return str;
        } else if (item.valueType == ValueType::INT){
            if (auto p = std::get_if<int>(&item.value)) {
                int tmpValue = *p;
                return @(tmpValue);
            }
        } else if (item.valueType == ValueType::LONG){
            if (auto p = std::get_if<long>(&item.value)) {
                long tmpValue = *p;
                return @(tmpValue);
            }
        } else if (item.valueType == ValueType::DOUBLE){
            if (auto p = std::get_if<double>(&item.value)) {
                double tmpValue = *p;
                return @(tmpValue);
            }
        } else if (item.valueType == ValueType::FLOAT){
            if (auto p = std::get_if<float>(&item.value)) {
                float tmpValue = *p;
                return @(tmpValue);
            }
        }
        return nullptr;
    }
    
    NSArray<NSObject *>* _Nonnull decodeMemoryContent() {
        NSMutableArray *array = [[NSMutableArray alloc] init];
        for (int i = 0; i < kvDataLen; i++) {
            NSObject *ocKey = objcValue(kvUserData[i].key);
            if (ocKey) {
                [array addObject:ocKey];
            }
            
            NSObject *ocValue = objcValue(kvUserData[i].value);
            if (ocValue) {
                [array addObject:ocValue];
            }
        }
        return array;
    }
    
    static NSArray<NSObject *>* _Nullable decodeFromFile(NSString * _Nonnull filePath, int maxCount = 0) {
        
        const char *cFilePath = filePath.UTF8String;
        std::string fullPath(cFilePath);
        
        std::string basePath, fileName;
        size_t found = fullPath.find_last_of("/");
        if (found != std::string::npos) {
            basePath = fullPath.substr(0, found);
            fileName = fullPath.substr(found + 1);
        } else {
            fileName = fullPath;
        }
        
        Reporter reporter;
        reporter.writer.openFile(fileName.c_str(), basePath.c_str(), O_RDONLY);
        
        auto list = reporter.writer.deserialize(maxCount);
        NSMutableArray *array = [[NSMutableArray alloc] init];
        for (auto &item : list) {
            if (item->valueType == ValueType::PTR) {
                NSString *str = [[NSString alloc] initWithBytes:item->getData() length:item->getSize() encoding:NSUTF8StringEncoding];
                if (str) {
                    [array addObject:str];
                    AAI_CRASH_CLOG("[%s]", item->toString().c_str());
                } else {
                    AAI_CRASH_CLOG("Unexpect item value[%s]", item->toString().c_str());
                }
            } else if (item->valueType == ValueType::INT){
                if (auto p = std::get_if<int>(&item->value)) {
                    int tmpValue = *p;
                    [array addObject:@(tmpValue)];
                }
            } else if (item->valueType == ValueType::LONG){
                if (auto p = std::get_if<long>(&item->value)) {
                    long tmpValue = *p;
                    [array addObject:@(tmpValue)];
                }
            } else if (item->valueType == ValueType::DOUBLE){
                if (auto p = std::get_if<double>(&item->value)) {
                    double tmpValue = *p;
                    [array addObject:@(tmpValue)];
                }
            } else if (item->valueType == ValueType::FLOAT){
                if (auto p = std::get_if<float>(&item->value)) {
                    float tmpValue = *p;
                    [array addObject:@(tmpValue)];
                }
            }
            
        }
        
        reporter.writer.closeFile();
        return array;
    }
};

Reporter* _Nonnull sharedReporter();


}};
#endif /* AAICrashReporter_hpp */
