//
//  AAIKeychain.h
//  AAILivenessSDK
//
//  Created by advance on 2023/7/10.
//  Copyright © 2023 Advance.ai. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <Security/Security.h>

NS_ASSUME_NONNULL_BEGIN

//
//  AAIKeychain.h
//  SSToolkit
//
//  Created by Sam Soffes on 5/19/10.
//  Copyright (c) 2009-2011 Sam Soffes. All rights reserved.
//

/** Error codes that can be returned in NSError objects. */
typedef enum {
    /** No error. */
    AAIKCErrorNone = noErr,
    
    /** Some of the arguments were invalid. */
    AAIKCErrorBadArguments = -1001,
    
    /** There was no password. */
    AAIKCErrorNoPassword = -1002,
    
    /** One or more parameters passed internally were not valid. */
    AAIKCErrorInvalidParameter = errSecParam,
    
    /** Failed to allocate memory. */
    AAIKCErrorFailedToAllocated = errSecAllocate,
    
    /** No trust results are available. */
    AAIKCErrorNotAvailable = errSecNotAvailable,
    
    /** Authorization/Authentication failed. */
    AAIKCErrorAuthorizationFailed = errSecAuthFailed,
    
    /** The item already exists. */
    AAIKCErrorDuplicatedItem = errSecDuplicateItem,
    
    /** The item cannot be found.*/
    AAIKCErrorNotFound = errSecItemNotFound,
    
    /** Interaction with the Security Server is not allowed. */
    AAIKCErrorInteractionNotAllowed = errSecInteractionNotAllowed,
    
    /** Unable to decode the provided data. */
    AAIKCErrorFailedToDecode = errSecDecode
} AAIKCErrorCode;

extern NSString *const kAAIKCErrorDomain;

/** Account name. */
extern NSString *const kAAIKCAccountKey;

/**
 Time the item was created.
 
 The value will be a string.
 */
extern NSString *const kAAIKCCreatedAtKey;

/** Item class. */
extern NSString *const kAAIKCClassKey;

/** Item description. */
extern NSString *const kAAIKCDescriptionKey;

/** Item label. */
extern NSString *const kAAIKCLabelKey;

/** Time the item was last modified.
 
 The value will be a string.
 */
extern NSString *const kAAIKCLastModifiedKey;

/** Where the item was created. */
extern NSString *const kAAIKCWhereKey;

/**
 Simple wrapper for accessing accounts, getting passwords, setting passwords, and deleting passwords using the system
 Keychain on Mac OS X and iOS.
 
 This was originally inspired by EMKeychain and SDKeychain (both of which are now gone). Thanks to the authors.
 AAIKeychain has since switched to a simpler implementation that was abstracted from [SSToolkit](http://sstoolk.it).
 */
@interface AAIKeychain : NSObject

///-----------------------
/// @name Getting Accounts
///-----------------------

/**
 Returns an array containing the Keychain's accounts for a given service, or `nil` if the Keychain doesn't have any
 accounts for the given service.
 
 @param serviceName The service for which to return the corresponding accounts.
 
 @param error If accessing the accounts fails, upon return contains an error that describes the problem.
 
 @return An array of dictionaries containing the Keychain's accountsfor a given `serviceName`, or `nil` if the Keychain
 doesn't have any accounts for the given `serviceName`. The order of the objects in the array isn't defined.
 
 @see accountsForService:
 */
+ (NSArray * _Nullable)accountsForService:(NSString *)serviceName error:(NSError **)error;


///------------------------
/// @name Getting Passwords
///------------------------


/**
 Returns a string containing the password for a given account and service, or `nil` if the Keychain doesn't have a
 password for the given parameters.
 
 @param serviceName The service for which to return the corresponding password.
 
 @param account The account for which to return the corresponding password.
 
 @param error If accessing the password fails, upon return contains an error that describes the problem.
 
 @return Returns a string containing the password for a given account and service, or `nil` if the Keychain doesn't
 have a password for the given parameters.
 
 @see passwordForService:account:
 */
+ (NSString * _Nullable)passwordForService:(NSString *)serviceName account:(NSString *)account error:(NSError **)error;

///------------------------
/// @name Setting Passwords
///------------------------

/**
 Sets a password in the Keychain.
 
 @param password The password to store in the Keychain.
 
 @param serviceName The service for which to set the corresponding password.
 
 @param account The account for which to set the corresponding password.
 
 @param error If setting the password fails, upon return contains an error that describes the problem.
 
 @return Returns `YES` on success, or `NO` on failure.
 
 @see setPassword:forService:account:
 */
+ (BOOL)aaiSetPassword:(NSString *)password forService:(NSString *)serviceName account:(NSString *)account error:(NSError **)error;

@end

NS_ASSUME_NONNULL_END
